SystemOrganization addCategory: #TusLibros!

TestCase subclass: #TusLibrosTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!TusLibrosTest methodsFor: 'instance creation' stamp: 'SebaTorres 6/12/2013 02:17'!
bookSales
	^BookSales withCatalog: self catalog.! !

!TusLibrosTest methodsFor: 'instance creation' stamp: 'SebaTorres 6/12/2013 03:09'!
cashier
	^Cashier withPriceList:  (self priceList) withMerchantProcessor: [ :x :y | ^true ]! !

!TusLibrosTest methodsFor: 'instance creation' stamp: 'SebaTorres 6/12/2013 02:16'!
catalog
	| catalog |
	catalog := Bag new.
	catalog add: 'book'.
	catalog add: 'anotherBook'.
	^catalog.! !

!TusLibrosTest methodsFor: 'instance creation' stamp: 'JuanManuelTastzian 5/29/2013 18:23'!
emptyCart
	^Cart newEmptyCart! !

!TusLibrosTest methodsFor: 'instance creation' stamp: 'SebaTorres 6/12/2013 02:15'!
interface
	^Interface withPatovica: self newSpartanCentinel withCashier: self cashier withClock: self now withBookSales: self bookSales.! !

!TusLibrosTest methodsFor: 'instance creation' stamp: 'SebaTorres 6/10/2013 16:20'!
newCreditCard
	^ CreditCard createWithExpirationDate: Date tomorrow andAmount:  100 andStatus: 'not stolen'.! !

!TusLibrosTest methodsFor: 'instance creation' stamp: 'SebaTorres 6/10/2013 12:43'!
newEmptyCreditCard
	^CreditCard createWithExpirationDate:  Date tomorrow andAmount:  0 andStatus: 'not stolen'.! !

!TusLibrosTest methodsFor: 'instance creation' stamp: 'SebaTorres 6/10/2013 12:38'!
newExpiredCreditCard
	^CreditCard createWithExpirationDate:  Date yesterday andAmount:  100 andStatus: 'not stolen'.! !

!TusLibrosTest methodsFor: 'instance creation' stamp: 'SebaTorres 6/10/2013 15:29'!
newSpartanCentinel
	^Spartan createWith: self userRegistrationData .! !

!TusLibrosTest methodsFor: 'instance creation' stamp: 'SebaTorres 6/10/2013 12:42'!
newStolenCreditCard
	^ CreditCard createWithExpirationDate: Date tomorrow andAmount:  100 andStatus: 'stolen'.! !

!TusLibrosTest methodsFor: 'instance creation' stamp: 'SebaTorres 6/10/2013 23:51'!
now
	^Clock withTime: TimeStamp now.! !

!TusLibrosTest methodsFor: 'instance creation' stamp: 'SebaTorres 6/11/2013 00:28'!
priceList
	| priceList |
	priceList := Dictionary new.
	priceList at: 'book' put: 1.
	priceList at: 'anotherBook' put: 3.
	^priceList.! !

!TusLibrosTest methodsFor: '5 - Session Test' stamp: 'SebaTorres 6/10/2013 23:53'!
test1After20MinutesSessionMustBeActive

	| session clock |

	clock := self now.
	session := Session withClient:  'ABC123' withCart:  self emptyCart withClock:  clock.
	
	clock addMinutes: 20.
	
	self deny:  session hasExpired. .
! !

!TusLibrosTest methodsFor: '1 - Cart Test' stamp: 'JuanManuelTastzian 5/29/2013 20:01'!
test1CartShouldBeEmptyAfterCreation
	self assert: self emptyCart isEmpty! !

!TusLibrosTest methodsFor: '6 - Interface Test' stamp: 'SebaTorres 6/11/2013 01:02'!
test1CreatingACartShouldReturnAnEmptyCart

	| interface cart cartID |
		
	interface := self interface.
	cartID := interface newCartOf: 'Sebas' withPassword: 'Torres'.
	
	cart := interface findCartByKey: cartID.

	self assert: cart isEmpty.! !

!TusLibrosTest methodsFor: '3 - Security Test' stamp: 'SebaTorres 6/11/2013 00:45'!
test1UserAuthenticationShouldReturnOkWithAuthorizedUser
	| id |
	id := self newSpartanCentinel validateUser: 'Juan' withPassword: 'Tastzian'.
	
	self assert: id > 0 .! !

!TusLibrosTest methodsFor: '2 - Cashier Test' stamp: 'JuanManuelTastzian 5/29/2013 21:19'!
test1cantCheckoutEmptyCart

	| cashier |
	cashier := Cashier new.
	
	self should: [cashier checkout: (self emptyCart) ]
		raise: Error 
		withExceptionDo: [ :anError | 
			self assert: anError description = Cashier cartIsEmpty ]! !

!TusLibrosTest methodsFor: '6 - Interface Test' stamp: 'SebaTorres 6/11/2013 01:19'!
test2AddingABookToAnExistentCartShouldUpdateIt

	| interface cart cartID |
		
	interface := self interface.
	cartID := interface newCartOf: 'Juan' withPassword: 'Tastzian'.
	
	cart := interface findCartByKey: cartID.

	self assert: cart isEmpty.
	
	interface addToCart: cartID theBook: 'book' .
	
	self assert: (cart numberOfUnitsAddedOf:  'book' ) = 1.! !

!TusLibrosTest methodsFor: '5 - Session Test' stamp: 'SebaTorres 6/10/2013 23:55'!
test2After30MinutesSessionMustNotBeActive

	| session clock |

	clock := self now.
	session := Session withClient:  'ABC123' withCart:  self emptyCart withClock:  clock.
	
	clock addMinutes: 31.
	
	self assert:  session hasExpired.
! !

!TusLibrosTest methodsFor: '1 - Cart Test' stamp: 'JuanManuelTastzian 5/29/2013 20:57'!
test2CartSizeShouldBeGreaterThan0AfterAddingABookToIt
	| cart book catalog |
	
	book := 'aBook'.
	catalog := Bag new.
	catalog add: book.
	cart := Cart withCatalog: catalog.
	
	cart add: book.
	
	self assert: (cart size > 0).! !

!TusLibrosTest methodsFor: '3 - Security Test' stamp: 'SebaTorres 6/10/2013 15:43'!
test2UserAuthenticationShouldReturnErrorWithInvalidUserPassword

	self should: [self newSpartanCentinel  validateUser: 'Juan' withPassword: 'Torres']
		 raise: Error withExceptionDo: [ :anError | 
			self assert: anError description = 'Error: User name or password invalid. Please try again.' ]! !

!TusLibrosTest methodsFor: '2 - Cashier Test' stamp: 'SebaTorres 6/10/2013 13:29'!
test2cashierChecksOutCorrectly

	"Este test se refiere a que el carrito hace bien el cobro de los libros"

	| cashier aBook catalog priceList anotherBook cart priceBook priceAnotherBook |

	aBook := 'aBook'.
	anotherBook := 'anotherBook'.

	catalog := Bag new.
	catalog add: aBook.
	catalog add: anotherBook .

	"Para crear un diccionario se le asignan asociaciones"
	priceBook := Association key: aBook value: 1.
	priceAnotherBook := Association key: anotherBook value: 3.

	priceList := Dictionary new.
	priceList add: priceBook.
	priceList add: priceAnotherBook.

	cashier := Cashier withPriceList: priceList.

	cart := Cart withCatalog: catalog.
	cart add: aBook. 
	cart add: aBook.
	cart add: anotherBook.
	
	self assert: (cashier checkout: cart) = 5. ! !

!TusLibrosTest methodsFor: '6 - Interface Test' stamp: 'SebaTorres 6/11/2013 01:34'!
test3AddingABookToAnInexistentCartShouldRaiseAnError

	| interface |
	
	interface := self interface.
	self should: [interface addToCart: 0 theBook: 'book' ]
	raise: Error
	withExceptionDo: [:anError | self assert: anError description = Interface cannotFindTheCart ]! !

!TusLibrosTest methodsFor: '2 - Cashier Test' stamp: 'JuanManuelTastzian 6/3/2013 20:17'!
test3BooksNotPresentInPriceListCantBeCheckedOut

	"Me fijo que los libros del carrito se encuentren en la lista de precios antes de hacer checkout"

	| cashier aBook catalog priceList anotherBook cart priceBook priceAnotherBook |

	aBook := 'aBook'.
	anotherBook := 'anotherBook'.

	catalog := Bag new.
	catalog add: aBook.
	catalog add: anotherBook .

	priceBook := Association key: aBook value: 1.
	priceAnotherBook := Association key: anotherBook value: 3.

	priceList := Dictionary new.
	priceList add: priceBook.

	cart := Cart withCatalog: catalog.
	cart add: aBook. 
	cart add: anotherBook.

	cashier := Cashier withPriceList: priceList.

	self should: [cashier checkout:  cart ]
		raise: Error 
		withExceptionDo: [ :anError | 
			self assert: anError description = Cashier bookNotInPriceList ]! !

!TusLibrosTest methodsFor: '5 - Session Test' stamp: 'SebaTorres 6/11/2013 00:03'!
test3RefreshingAnActiveSessionShouldUpdateTheLastTimeAccessed

	| clock session newTime |

	clock := self now.
	session := Session withClient: 'ABC123' withCart: self emptyCart withClock: clock.
	
	clock addMinutes: 10.
	
	newTime := clock time.

 	self deny: session lastTimeAccessed = newTime.
	session refresh.
	self assert: session lastTimeAccessed equals: newTime.
! !

!TusLibrosTest methodsFor: '3 - Security Test' stamp: 'SebaTorres 6/10/2013 15:44'!
test3UserAuthenticationShouldReturnErrorWithInvalidUserName

	self should: [self newSpartanCentinel  validateUser: 'Roberto' withPassword: 'Tastzian']
		 raise: Error withExceptionDo: [ :anError | 
			self assert: anError description = 'Error: User name or password invalid. Please try again.' ]! !

!TusLibrosTest methodsFor: '1 - Cart Test' stamp: 'JuanManuelTastzian 5/29/2013 20:59'!
test3aBookShouldBeListedAfterBeingAddedToACart
	| cart book catalog |
	
	book := 'aBook'.
	catalog := Bag new.
	catalog add: book.
	cart := Cart withCatalog: catalog.
	
	cart add: book.
	self assert: (cart includes: book)! !

!TusLibrosTest methodsFor: '6 - Interface Test' stamp: 'SebaTorres 6/12/2013 01:45'!
test4AddingABookToAnExpiredCartSessionShouldRaiseAnError
	
	| interface clock cartID |
	interface := self interface.
	clock := interface clock.

	cartID := interface newCartOf: 'Juan' withPassword: 'Tastzian'.
	
	clock addMinutes: 40.
	
	self	should: [ interface addToCart: cartID theBook: 'book' ]
		raise: Error
		withExceptionDo: [ :anError | self assert: anError description = Interface sessionExpired ]! !

!TusLibrosTest methodsFor: '2 - Cashier Test' stamp: 'SebaTorres 6/10/2013 12:34'!
test4CashierCantCheckOutExpiredCard

	| aBook catalog priceBook priceList cart cashier expiredCreditCard aMerchantProcessor |
	aBook := 'aBook'.

	catalog := Bag new.
	catalog add: aBook.
	
	priceBook := Association key: aBook value: 1.
	
	priceList := Dictionary new.
	priceList add: priceBook.
	
	cart := Cart withCatalog: catalog.
	cart add: aBook. 
	
	aMerchantProcessor := PlagableMerchantProcessor withValidator: [ :anAmount :aCreditCard | 
		Error signal: 'Cannot checkout payment with expired credit card' ].
	
	cashier := Cashier withPriceList: priceList withMerchantProcessor: aMerchantProcessor.
	
	expiredCreditCard := self newExpiredCreditCard.
	
	self should: [cashier checkout: cart with: expiredCreditCard ]
		raise: Error 
		withExceptionDo: [ :anError | 
			self assert: anError description = Cashier expiredCreditCard ].! !

!TusLibrosTest methodsFor: '1 - Cart Test' stamp: 'JuanManuelTastzian 5/29/2013 20:59'!
test4aCartHasOnlyOneBookAfterAddingABookToAnEmptyCart
	| cart book catalog |
	
	book := 'aBook'.
	catalog := Bag new.
	catalog add: book.
	cart := Cart withCatalog: catalog.
	
	cart add: book.
	self assert: (cart includes: book).
	self assert: cart size = 1! !

!TusLibrosTest methodsFor: '6 - Interface Test' stamp: 'SebaTorres 6/12/2013 01:55'!
test5AttempingCheckOutAnExpiredCartShouldRaiseAnError
	
	| interface clock aCC cartID |
	interface := self interface.
	clock := interface clock.
	aCC := self newCreditCard .

	cartID := interface newCartOf: 'Sebas' withPassword: 'Torres'.
	
	clock addMinutes: 42.
	
	self	should: [ interface checkOut: cartID  withPaymentType: aCC ]
		raise: Error
		withExceptionDo: [ :anError | self assert: anError description =  Interface sessionExpired  ]! !

!TusLibrosTest methodsFor: '2 - Cashier Test' stamp: 'SebaTorres 6/10/2013 12:42'!
test5CashierCantDebitFromEmptyBankAccount

	| aBook catalog priceBook priceList cart cashier emptyCreditCard aMerchantProcessor |
	aBook := 'aBook'.

	catalog := Bag new.
	catalog add: aBook.
	
	priceBook := Association key: aBook value: 1.
	
	priceList := Dictionary new.
	priceList add: priceBook.
	
	cart := Cart withCatalog: catalog.
	cart add: aBook. 
	
	aMerchantProcessor := PlagableMerchantProcessor withValidator: [ :anAmount :aCreditCard | 
		Error signal: 'Cannot checkout payment with an empty credit card' ].
	
	cashier := Cashier withPriceList: priceList withMerchantProcessor: aMerchantProcessor.
	
	emptyCreditCard := self newEmptyCreditCard.
	
	self should: [cashier checkout: cart with: emptyCreditCard ]
		raise: Error 
		withExceptionDo: [ :anError | 
			self assert: anError description = Cashier emptyCreditCard ]! !

!TusLibrosTest methodsFor: '1 - Cart Test' stamp: 'JuanManuelTastzian 5/29/2013 21:02'!
test5aBookCanBeAddedMoreThanOnce
	| cart book anotherBook catalog |
	
	book := 'aBook'.
	anotherBook := 'anotherBook'.
	catalog := Bag new.
	catalog add: book.
	catalog add: anotherBook.
	
	cart := Cart withCatalog: catalog.
	
	cart add: book.
	cart add: book.
	cart add: anotherBook.
	
	self assert: (cart includes: book).
	self assert: cart size = 3.
	self assert: (cart numberOfUnitsAddedOf: book) = 2.
	self assert: (cart numberOfUnitsAddedOf: anotherBook) = 1.! !

!TusLibrosTest methodsFor: '6 - Interface Test' stamp: 'SebaTorres 6/12/2013 02:38'!
test6CreateingANewCartReturnsANewEmptyCart
	
	| interface cartID cart |
	interface := self interface.
	
	cartID := interface newCartOf: 'Juan' withPassword: 'Tastzian'.
	
	cart := interface findCartByKey: cartID.

	self assert: cart isEmpty.! !

!TusLibrosTest methodsFor: '1 - Cart Test' stamp: 'SebaTorres 6/10/2013 12:59'!
test6booksThatAreNotOnTheCatalogCantBeAddedToCart
	| cart book |
	
	cart := self emptyCart.
	book := 'aBook'.
	
	self should: [cart add: book ]
		raise: Error 
		withExceptionDo: [ :anError | 
			self assert: anError description = Cart productNotInCatalog ]! !

!TusLibrosTest methodsFor: '2 - Cashier Test' stamp: 'SebaTorres 6/10/2013 12:40'!
test6cashierCantCheckoutStolenCreditCard

	| aBook catalog priceBook priceList cart cashier stolenCreditCard aMerchantProcessor |
	aBook := 'aBook'.

	catalog := Bag new.
	catalog add: aBook.
	
	priceBook := Association key: aBook value: 1.
	
	priceList := Dictionary new.
	priceList add: priceBook.
	
	cart := Cart withCatalog: catalog.
	cart add: aBook. 
	
	aMerchantProcessor := PlagableMerchantProcessor withValidator: [ :anAmount :aCreditCard | 
		Error signal: 'Cannot checkout payment with a stolen credit card. Calling the cops' ].
	
	cashier := Cashier withPriceList: priceList withMerchantProcessor: aMerchantProcessor.
	
	stolenCreditCard := self newStolenCreditCard.
	
	self should: [cashier checkout: cart with: stolenCreditCard ]
		raise: Error 
		withExceptionDo: [ :anError | 
			self assert: anError description = Cashier stolenCreditCard ]! !

!TusLibrosTest methodsFor: '6 - Interface Test' stamp: 'SebaTorres 6/12/2013 02:43'!
test7ShowingACartShouldShowTheItemsCart

	| interface cartID cart |
	interface := self interface.
	cartID := interface newCartOf: 'Sebas' withPassword: 'Torres'.

	interface addToCart: cartID  theBook: 'book'. 
	interface addToCart: cartID  theBook: 'anotherBook'. 

	cart := interface findCartByKey: cartID.
	
	self assert: (interface showCart: cartID) = cart books.! !

!TusLibrosTest methodsFor: '2 - Cashier Test' stamp: 'SebaTorres 6/10/2013 16:39'!
test7TicketOfCheckoutHasTheSameBooksAndQuantitiesThanCart

	| aBook catalog priceBook priceList cart cashier aMerchantProcessor okCreditCard ticket ticketItems cartItems |
	aBook := 'aBook'.

	catalog := Bag new.
	catalog add: aBook.
	
	priceBook := Association key: aBook value: 1.
	
	priceList := Dictionary new.
	priceList add: priceBook.
	
	cart := Cart withCatalog: catalog.
	cart add: aBook. 
	
	aMerchantProcessor := PlagableMerchantProcessor withValidator: [ :anAmount :aCreditCard | ^true ].
	
	cashier := Cashier withPriceList: priceList withMerchantProcessor: aMerchantProcessor.
	
	 okCreditCard := self newCreditCard .
	
	ticket := cashier checkout: cart with: okCreditCard.
	ticketItems := ticket items.
	cartItems := cart books.

	self assert: [ ticket items size = cart books size ].
	self assert: ( ticketItems allSatisfy: [ :book | (ticketItems occurrencesOf: book) = (cartItems occurrencesOf: book )]).! !

!TusLibrosTest methodsFor: '1 - Cart Test' stamp: 'JuanManuelTastzian 5/29/2013 20:39'!
test7onlyBooksFromCatalogCanBeAddedToCart
	| cart book catalog |
	
	book := 'aBook'.
	catalog := Bag new.
	catalog add: book.
	cart := Cart withCatalog: catalog.
	
	cart add: book.
	
	self assert: (cart includes: book).
	self assert: cart size = 1.! !

!TusLibrosTest methodsFor: '6 - Interface Test' stamp: 'SebaTorres 6/12/2013 02:52'!
test8ShowingAnInexistentCartShouldRaiseError

	| interface |
	
	interface := self interface.
	
	self should: [interface showCart: 0]
	raise: Error
	withExceptionDo: [:anError | self assert: anError description = Interface cannotFindTheCart ]! !

!TusLibrosTest methodsFor: '6 - Interface Test' stamp: 'SebaTorres 6/12/2013 03:05'!
test9OrderedItemsAreListedByAValidID
	
	| interface cartID session id orderedItems |
	interface := self interface.
	
	cartID := interface newCartOf: 'Sebas' withPassword: 'Torres' .
	session := interface getSessionByKey: cartID.
	id := session clientID.
	
	interface addToCart: cartID theBook: 'book'.
	interface checkOut: cartID withPaymentType: self newCreditCard.

	orderedItems := interface getOrderedItems: id.

	self deny: orderedItems isEmpty.! !

!TusLibrosTest methodsFor: 'instance creation' stamp: 'SebaTorres 6/10/2013 15:29'!
userRegistrationData
	| registry |
	registry := Dictionary new.
	registry at: 'Sebas' put: 'Torres'.
	registry at: 'Juan' put: 'Tastzian'.
	^registry ! !

Object subclass: #BookSales
	instanceVariableNames: 'catalog sales'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!BookSales class methodsFor: 'instance creation' stamp: 'SebaTorres 6/12/2013 02:18'!
withCatalog: aCatalog 
	^self new initializeWithCatalog: aCatalog.! !

!BookSales methodsFor: 'accessing' stamp: 'SebaTorres 6/12/2013 02:27'!
catalog
	^catalog! !

!BookSales methodsFor: 'accessing' stamp: 'SebaTorres 6/12/2013 03:17'!
initializeWithCatalog: aCatalog
	catalog := aCatalog.! !

Object subclass: #Cart
	instanceVariableNames: 'books catalog'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

Cart class
	instanceVariableNames: 'size'!

Cart class
	instanceVariableNames: 'size'!

!Cart class methodsFor: 'class initialization' stamp: 'JuanManuelTastzian 5/29/2013 18:30'!
newEmptyCart
	^self new withoutBooks! !

!Cart class methodsFor: 'error handling' stamp: 'JuanManuelTastzian 5/29/2013 20:54'!
productNotInCatalog
	^ 'Error: The requested product is not present in the catalog'.! !

!Cart class methodsFor: 'class initialization' stamp: 'JuanManuelTastzian 5/29/2013 20:41'!
withCatalog: aCatalog 
	^ self new catalog: aCatalog.! !

!Cart methodsFor: 'accessing' stamp: 'JuanManuelTastzian 5/29/2013 20:53'!
add: aBook 
	(catalog includes: aBook) ifTrue: [books add: aBook] ifFalse: [ Error signal: 'The requested product is not present in the catalog' ]
	! !

!Cart methodsFor: 'accessing' stamp: 'JuanManuelTastzian 5/29/2013 18:36'!
books
	^ books! !

!Cart methodsFor: 'accessing' stamp: 'JuanManuelTastzian 5/29/2013 20:43'!
catalog: aCatalog 
	books := Bag new.
	catalog := aCatalog.! !

!Cart methodsFor: 'accessing' stamp: 'JuanManuelTastzian 5/29/2013 19:16'!
includes: aBook 
	^books includes: aBook! !

!Cart methodsFor: 'accessing' stamp: 'JuanManuelTastzian 5/29/2013 20:04'!
isEmpty
	^ self size = 0! !

!Cart methodsFor: 'accessing' stamp: 'JuanManuelTastzian 5/29/2013 19:26'!
numberOfUnitsAddedOf: aBook 
	^(books occurrencesOf: aBook)! !

!Cart methodsFor: 'as yet unclassified' stamp: 'JuanManuelTastzian 5/29/2013 18:38'!
size
	^books size! !

!Cart methodsFor: 'accessing' stamp: 'JuanManuelTastzian 5/29/2013 20:27'!
withoutBooks
	books := Bag new.
	catalog := Bag new.! !

Object subclass: #Cashier
	instanceVariableNames: 'priceList merchantProcessor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Cashier class methodsFor: 'message handling' stamp: 'JuanManuelTastzian 6/3/2013 19:12'!
bookNotInPriceList
	^ 'Error: Cannot checkout books that are not in the price list'! !

!Cashier class methodsFor: 'error handling' stamp: 'JuanManuelTastzian 5/29/2013 21:20'!
cartIsEmpty
	^ 'Error: Cannot checkout an empty cart'.! !

!Cashier class methodsFor: 'as yet unclassified' stamp: 'JuanManuelTastzian 6/5/2013 17:15'!
emptyCreditCard
	^ 'Error: Cannot checkout payment with an empty credit card'! !

!Cashier class methodsFor: 'error handling' stamp: 'JuanManuelTastzian 6/3/2013 20:36'!
expiredCreditCard
	^ 'Error: Cannot checkout payment with expired credit card'! !

!Cashier class methodsFor: 'as yet unclassified' stamp: 'JuanManuelTastzian 6/5/2013 17:43'!
stolenCreditCard
	^ 'Error: Cannot checkout payment with a stolen credit card. Calling the cops'! !

!Cashier class methodsFor: 'instance creation' stamp: 'SebaTorres 6/3/2013 02:54'!
withPriceList: aPriceList 
	^self new withPriceList: aPriceList.! !

!Cashier class methodsFor: 'instance creation' stamp: 'JuanManuelTastzian 6/5/2013 20:51'!
withPriceList: aPriceList withMerchantProcessor: aPlagableMerchantProcessor 
	^self new withPriceList: aPriceList withMerchantProcessor: aPlagableMerchantProcessor.! !

!Cashier methodsFor: 'as yet unclassified' stamp: 'SebaTorres 6/3/2013 03:24'!
checkout: aCart
	(aCart isEmpty) ifTrue: [ Error signal: 'Cannot checkout an empty cart' ].
	(aCart books asSet = priceList keys asSet) ifFalse: [ Error signal: 'Cannot checkout books that are not in the price list' ].
	^aCart books detectSum: [ :book | priceList at: book ].! !

!Cashier methodsFor: 'as yet unclassified' stamp: 'SebaTorres 6/12/2013 03:21'!
checkout: aCart with: aCreditCard
	| amount |
	"probar mejor con un allSatisfy para ver si los libros del carrito estan en la lista de precios!!"
	(aCart isEmpty) ifTrue: [ Error signal: 'Cannot checkout an empty cart' ].
	(aCart books asSet = priceList keys asSet) ifFalse: [ Error signal: 'Cannot checkout books that are not in the price list' ].
	
	amount := aCart books detectSum: [ :book | priceList at: book ].
	merchantProcessor debit: amount from: aCreditCard.
	
	^Ticket of: aCart payedWith: aCreditCard withTotal: amount withItems: aCart books
	
	"(aCreditCard isExpired) ifTrue: [ Error signal: 'Cannot checkout payment with expired credit card' ].
	^aCart books detectSum: [ :book | priceList at: book ]."! !

!Cashier methodsFor: 'accessing' stamp: 'SebaTorres 6/3/2013 02:56'!
withPriceList: aPriceList 
	priceList := aPriceList.! !

!Cashier methodsFor: 'accessing' stamp: 'JuanManuelTastzian 6/5/2013 20:52'!
withPriceList: aPriceList withMerchantProcessor: aPlagableMerchantProcessor 
	priceList := aPriceList.
	merchantProcessor := aPlagableMerchantProcessor.! !

Object subclass: #Clock
	instanceVariableNames: 'time'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Clock class methodsFor: 'as yet unclassified' stamp: 'SebaTorres 6/10/2013 17:52'!
withTime: aTime
	^self new initializeTime: aTime.! !

!Clock methodsFor: 'as yet unclassified' stamp: 'SebaTorres 6/10/2013 18:00'!
addMinutes: minutes 
	self updateTime: (time plusSeconds: (minutes * 60))! !

!Clock methodsFor: 'as yet unclassified' stamp: 'SebaTorres 6/10/2013 17:52'!
initializeTime: aTime
	time := aTime! !

!Clock methodsFor: 'as yet unclassified' stamp: 'SebaTorres 6/10/2013 17:57'!
time
	^time! !

!Clock methodsFor: 'as yet unclassified' stamp: 'SebaTorres 6/10/2013 17:56'!
updateTime: aTime
	time := aTime.! !

Object subclass: #CreditCard
	instanceVariableNames: 'expireDate moneyLeft status'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!CreditCard class methodsFor: 'class initialization' stamp: 'SebaTorres 6/10/2013 12:20'!
createWithExpirationDate: anExpirationDate andAmount: anAmount andStatus: aStatus
	^self new newWith: anExpirationDate andAmount:  anAmount andStatus: aStatus! !

!CreditCard methodsFor: 'accessing' stamp: 'JuanManuelTastzian 6/5/2013 16:40'!
expireDate
	^expireDate! !

!CreditCard methodsFor: 'accessing' stamp: 'JuanManuelTastzian 6/5/2013 17:19'!
isEmpty
	^ moneyLeft <= 0! !

!CreditCard methodsFor: 'accessing' stamp: 'SebaTorres 6/10/2013 01:58'!
isExpired: aDate
	^ expireDate < aDate.! !

!CreditCard methodsFor: 'accessing' stamp: 'JuanManuelTastzian 6/5/2013 17:41'!
isStolen
	^status = 'stolen'! !

!CreditCard methodsFor: 'accessing' stamp: 'JuanManuelTastzian 6/5/2013 17:19'!
moneyLeft
	^moneyLeft! !

!CreditCard methodsFor: 'accessing' stamp: 'SebaTorres 6/10/2013 12:14'!
newWith: anExpirationDate andAmount: anAmount andStatus: aStatus
	expireDate := anExpirationDate .
	moneyLeft := anAmount .
	status := aStatus .! !

Object subclass: #Interface
	instanceVariableNames: 'patovica cashier clock sessions salesBook'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Interface class methodsFor: 'error handling' stamp: 'SebaTorres 6/11/2013 01:34'!
cannotFindTheCart
	^'Error: Cannot find the cart'! !

!Interface class methodsFor: 'error handling' stamp: 'SebaTorres 6/12/2013 01:48'!
sessionExpired
	^'Error: Session has expired, please log in again.'! !

!Interface class methodsFor: 'instance creation' stamp: 'SebaTorres 6/11/2013 00:30'!
withPatovica: aSpartan withCashier: aCashier withClock: aClock 
	 ^self new initializeWith: aSpartan withCashier: aCashier withClock: aClock.! !

!Interface class methodsFor: 'instance creation' stamp: 'SebaTorres 6/12/2013 02:22'!
withPatovica: aSpartan withCashier: aCashier withClock: aClock withBookSales: aBookSales 
	^self new initializeWith: aSpartan withCashier: aCashier withClock: aClock withBookSales: aBookSales.! !

!Interface methodsFor: 'associating' stamp: 'SebaTorres 6/11/2013 01:29'!
addToCart: aCartID theBook: aBook
	| session cart |
	session := self getSessionByKey: aCartID.
	
	(session hasExpired ) ifTrue: [ Error signal: 'Session has expired, please log in again.' ].
	
	cart := self findCartByKey: aCartID.
	
	cart add: aBook.
	
	session refresh.
! !

!Interface methodsFor: 'as yet unclassified' stamp: 'SebaTorres 6/12/2013 02:29'!
checkOut: aCartID withPaymentType: aCreditCard

       | session cart sale clientID  |
	session := self getSessionByKey: aCartID.

	(session hasExpired) ifTrue: [ ^Error signal: 'Session has expired, please log in again.' ].

	cart := self findCartByKey: aCartID.
	
	sale := cashier checkout: cart with: aCreditCard.
	
	clientID := session clientID.
		
	salesBook addSale: sale for: clientID.! !

!Interface methodsFor: 'accessing' stamp: 'SebaTorres 6/12/2013 01:41'!
clock
	^ clock! !

!Interface methodsFor: 'accessing' stamp: 'SebaTorres 6/11/2013 01:07'!
findCartByKey: aCartID 
	| session |
	session := self getSessionByKey: aCartID.
	^session cart.! !

!Interface methodsFor: 'accessing' stamp: 'SebaTorres 6/11/2013 01:24'!
getSessionByKey: aCartID 
	((sessions includesKey: aCartID)) ifFalse: [^Error signal: 'Cannot find the cart'].
	^(sessions at: aCartID )! !

!Interface methodsFor: 'private' stamp: 'SebaTorres 6/12/2013 02:26'!
initializeWith: aSpartan withCashier: aCashier withClock: aClock 
	patovica := aSpartan.
	cashier := aCashier.
	clock := aClock.
	sessions := Dictionary new.! !

!Interface methodsFor: 'accessing' stamp: 'SebaTorres 6/12/2013 02:25'!
initializeWith: aSpartan withCashier: aCashier withClock: aClock withBookSales: aBookSales 
	patovica := aSpartan.
	cashier := aCashier.
	clock := aClock.
	salesBook := aBookSales.
	sessions := Dictionary new.! !

!Interface methodsFor: 'associating' stamp: 'SebaTorres 6/12/2013 03:07'!
newCartOf: aUser withPassword: aPassword 
	| id cart newSession |
	id := patovica validateUser:  aUser withPassword: aPassword.
	cart := Cart withCatalog: (salesBook catalog).
	newSession := Session withClient: aUser withCart: cart withClock: clock.

	sessions at: id put: newSession.

	^id! !

!Interface methodsFor: 'accessing' stamp: 'SebaTorres 6/12/2013 02:47'!
showCart: aCartID 
	
	| session cart |
	session := self getSessionByKey: aCartID.
	
	(session hasExpired) ifTrue: [ ^Error signal: 'Session has expired, please log in again.' ].
	
	cart := self findCartByKey: aCartID.
	
	session refresh.
	
	^cart books.! !

Object subclass: #PlagableMerchantProcessor
	instanceVariableNames: 'blockValidator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!PlagableMerchantProcessor class methodsFor: 'class initialization' stamp: 'SebaTorres 6/10/2013 13:09'!
withValidator: aBlockValidator 
	^ self new validator: aBlockValidator.! !

!PlagableMerchantProcessor methodsFor: 'accessing' stamp: 'JuanManuelTastzian 6/5/2013 21:17'!
debit: anAmount from: aCreditCard
	^ blockValidator value: anAmount value: aCreditCard.! !

!PlagableMerchantProcessor methodsFor: 'accessing' stamp: 'SebaTorres 6/10/2013 13:09'!
validator: aBlockValidator
	blockValidator := aBlockValidator.! !

Object subclass: #Session
	instanceVariableNames: 'clientID cart clock lastTimeAccessed'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Session class methodsFor: 'as yet unclassified' stamp: 'SebaTorres 6/10/2013 18:35'!
withClient: aClientID withCart: aCart withClock: aClock 
	^self new initializeWith: aClientID andCart: aCart andClock: aClock.! !

!Session methodsFor: 'accessing' stamp: 'SebaTorres 6/10/2013 22:54'!
cart
	^cart! !

!Session methodsFor: 'accessing' stamp: 'SebaTorres 6/10/2013 22:54'!
clientID
	^clientID! !

!Session methodsFor: 'accessing' stamp: 'SebaTorres 6/10/2013 23:00'!
clock
	^clock! !

!Session methodsFor: 'comparing' stamp: 'SebaTorres 6/10/2013 19:03'!
hasExpired
	
	| thirtyMinutesToSeconds |
	
	thirtyMinutesToSeconds := 60 * 30.
	
	^(lastTimeAccessed plusSeconds: thirtyMinutesToSeconds ) <= (clock time)! !

!Session methodsFor: 'initialize-release' stamp: 'SebaTorres 6/10/2013 18:47'!
initializeWith: aClientID andCart: aCart andClock: aClock 
	clientID := aClientID.
	cart := aCart.
	clock := aClock.
	lastTimeAccessed := clock time! !

!Session methodsFor: 'accessing' stamp: 'SebaTorres 6/11/2013 00:04'!
lastTimeAccessed
	^ lastTimeAccessed! !

!Session methodsFor: 'accessing' stamp: 'SebaTorres 6/10/2013 22:53'!
lastTimeAccessed: aTime
	lastTimeAccessed := aTime! !

!Session methodsFor: 'refreshing' stamp: 'SebaTorres 6/10/2013 22:52'!
refresh
	lastTimeAccessed := clock time! !

Object subclass: #Spartan
	instanceVariableNames: 'users'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Spartan class methodsFor: 'as yet unclassified' stamp: 'SebaTorres 6/10/2013 15:10'!
createWith: aUserRegistry
	^self new registry: aUserRegistry.! !

!Spartan methodsFor: 'as yet unclassified' stamp: 'SebaTorres 6/10/2013 15:10'!
registry: aUserRegistry
	users := aUserRegistry .! !

!Spartan methodsFor: 'as yet unclassified' stamp: 'SebaTorres 6/11/2013 00:44'!
validateUser: aUserID withPassword: aPassword
	((users includesKey: aUserID ) and: [(users at: aUserID ) = aPassword ]) ifTrue: [^Random new nextInt: 1000].
	^ Error signal: 'User name or password invalid. Please try again.'! !

Object subclass: #Ticket
	instanceVariableNames: 'cart paymentEntity total items'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Ticket class methodsFor: 'as yet unclassified' stamp: 'SebaTorres 6/10/2013 14:07'!
of: aCart payedWith: aCreditCard withTotal: amount withItems: cartItems 
	^self new cart: aCart payedWith: aCreditCard withTotal: amount withItems: cartItems.! !

!Ticket methodsFor: 'as yet unclassified' stamp: 'SebaTorres 6/10/2013 14:08'!
cart: aCart payedWith: aCreditCard withTotal: amount withItems: cartItems
	cart := aCart.
	paymentEntity := aCreditCard.
	total := amount.
	items := cartItems.! !
